@@ -8,6 +8,7 @@ from collections import defaultdict
 
 from littlepython.ast import SetArrayItem, GetArrayItem
 from littlepython.parser import Assign, Block, ControlBlock, If, BinaryOp, UnaryOp
+from littlepython.feature import Features
 
 
 def defaultdict_to_list(d):
@@ -19,6 +20,15 @@ def defaultdict_to_list(d):
             l[k] = v
     return l
 
+
+class AlreadyRunningException(Exception):
+    pass
+
+
+class ExecutionCountExceededException(Exception):
+    pass
+
+
 class SymbolTable(object):
     def resolve(self, name):
         raise NotImplementedError("Must be implemented")
@@ -49,6 +59,7 @@ class GlobalSymbolTable(SymbolTable):
 
 
 class LPProg(object):
+    # TODO: allow multithreaded execution.
     binaryOps = {"+": lambda a, b: a + b,
                  "-": lambda a, b: a - b,
                  "*": lambda a, b: a * b,
@@ -67,10 +78,16 @@ class LPProg(object):
                 "+": lambda a: a,
                 "-": lambda a: -a}
 
-    def __init__(self, ast):
+    def __init__(self, ast, features):
         self.ast = ast
+        self.features = features
+        self.running = False
+        self.count_remaining = -1
 
     def handle_var(self, node, sym_tbl):
+        if node.value == "rand" and Features.RANDOM_VAR in self.features:
+            from random import randint
+            return randint(-2147483647, 2147483647)
         return sym_tbl[node.value]
 
     def handle_int(self, node, sym_tbl):
@@ -113,13 +130,20 @@ class LPProg(object):
         var[self.handle(node.right, sym_tbl)] = self.handle(node.expr, sym_tbl)
 
     def handle(self, node, sym_tbl):
+        self.count_remaining -= 1
+        if self.count_remaining == 0:
+            raise ExecutionCountExceededException("Running this program would require more operations than allowed.")
         name = 'handle_' + type(node).__name__.lower()
         handler = getattr(self, name, None)
         if handler is None:
             raise NotImplementedError("No {} found.".format(name))
         return handler(node, sym_tbl)
 
-    def run(self, static_vars=None):
+    def run(self, static_vars=None, max_op_count=-1):
+        if self.running:
+            raise AlreadyRunningException("This program can only be run on one thread. And this one is already running.")
+        self.running = True
+        self.count_remaining = max_op_count
         state = {}
         if static_vars is not None:
             for key, var in static_vars.items():
@@ -139,4 +163,5 @@ class LPProg(object):
             else:
                 end_state[key] = var
 
+        self.running = False
         return end_state
